==> libggi_1.99.2.0b2.1.orig.tar.gz: libggi-2.0b2.1/display/Xlib/visual.c <==
/* $Id: visual.c,v 1.31 1999/02/13 01:52:23 marcus Exp $
******************************************************************************

   Display-Xlib initialization.

   Copyright (C) 1995 Andreas Beck      [becka@ggi-project.org]
   Copyright (C) 1997 Jason McMullan    [jmcc@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdlib.h>

#include <ggi/internal/ggi-dl.h>
#include <ggi/display/xlib.h>

#define NUM_OPTS 2

static gg_option x_options[NUM_OPTS] =
{
	{ "inroot", "no" },
	{ "inwin",  "no" }
};


static inline Cursor make_cursor(Display *disp)
{
	char data[] = { 0xf8, 0xfa, 0xf8 };
	char mask[] = { 0xfa, 0xff, 0xfa };
	Pixmap crsrpix, crsrmask;
	XColor black = { 0, 0x0, 0x0, 0x0 },
	       white = { 0, 0xffff, 0xffff, 0xffff };
	Cursor mycrsr;

	crsrpix = XCreateBitmapFromData(disp, DefaultRootWindow(disp),
					data, 3, 3);
	crsrmask = XCreateBitmapFromData(disp, DefaultRootWindow(disp),
					mask, 3, 3);
	mycrsr = XCreatePixmapCursor(disp, crsrpix, crsrmask,
				     &black, &white, 1, 1);
	XFreePixmap(disp, crsrpix);
	XFreePixmap(disp, crsrmask);

	return mycrsr;
}


int GGIdlinit(ggi_visual *vis, const char *args, void *argptr)
{
	struct Xhooks *priv;
	Display *disp;
	void *lock;

	if (args) {
		args = ggParseOptions((char *)args, x_options,
				      NUM_OPTS);
		if (args == NULL) {
			fprintf(stderr, "display-xlib: error in arguments.\n");
			return GGI_DL_ERROR;
		}
	}

	GGIDPRINT_MISC("Xlib-target wants display %s\n", args);
	disp = XOpenDisplay(args);
	if (disp == NULL) return GGI_DL_ERROR;
	GGIDPRINT_MISC("Xlib: has display %s\n", args);

	if ((lock = ggLockCreate()) == NULL) {
		XCloseDisplay(disp);
		return GGI_DL_ERROR;
	}

	priv = _ggi_malloc(sizeof(struct Xhooks));

	priv->display = disp;
	priv->window = RootWindow(priv->display, priv->screen);
	priv->cmap = 0;
	priv->gc = 0;
	priv->tempgc = 0;
	priv->wintype = GGIX_NORMAL;
	priv->XLibLock = lock;

	priv->screen = DefaultScreen(priv->display);
	GGIDPRINT_MISC("Xlib: has screen %d\n", priv->screen);

	/* Get size of root window */
	{
		Window dummywin;
		int dummy;
		unsigned int w, h, udummy;

		XGetGeometry(disp, DefaultRootWindow(disp), &dummywin,
			     &dummy, &dummy, &w, &h,
			     &udummy, &udummy);
		if (w > 640) w = 640;
		if (h > 480) h = 480;
		priv->defsize.x = w;
		priv->defsize.y = h;
	}

	if (x_options[0].result[0] != 'n') {
		priv->wintype = GGIX_ROOT;
		GGIDPRINT_MISC("Xlib: using root window\n");
	} else if (x_options[1].result[0] != 'n') {
		priv->wintype = GGIX_WIN;
		priv->window = strtol(x_options[1].result, NULL, 0);
		GGIDPRINT_MISC("Xlib: using window id 0x%x\n", priv->window);
	} else {
		priv->wintype = GGIX_NORMAL;
	}

	priv->cursor = make_cursor(disp);

	/* This doesn't work yet... */
#if 0
	if ((priv->textfont = XLoadQueryFont(disp, "fixed")) != NULL) {
		GGIDPRINT_MISC("Xlib: using font with "
			       "dimension %dx%d\n",
			       priv->textfont->max_bounds.width,
			       priv->textfont->max_bounds.ascent
			       + priv->textfont->max_bounds.descent);
	}
#else
	priv->textfont = NULL;
#endif

	LIBGGI_PRIVATE(vis) = priv;

	/* Has mode management */
	vis->opdisplay->flush=GGI_Xlib_flush;
	vis->opdisplay->getmode=GGI_Xlib_getmode;
	vis->opdisplay->setmode=GGI_Xlib_setmode;
	vis->opdisplay->checkmode=GGI_Xlib_checkmode;
	vis->opdisplay->getapi =GGI_Xlib_getapi;
	vis->opdisplay->setflags=GGI_Xlib_setflags;

	{
		gii_inputxwin_arg args;
		gii_input *inp;
		
		args.disp = priv->display;
		args.ptralwaysrel = 0;
		args.wait = 1;
		args.exposefunc = args.exposearg = NULL;
		args.gglock = lock;
		
		if ((inp = giiOpen("xwin", &args, NULL)) == NULL) {
			GGIDPRINT_MISC("Unable to open xwin inputlib\n");
			GGIdlcleanup(vis);
			return GGI_DL_ERROR;
		}

		priv->inp = inp;
		/* Now join the new event source in. */
		vis->input = giiJoinInputs(vis->input, inp);
        }

	LIBGGI_GC(vis) = _ggi_malloc(sizeof(ggi_gc));

	GGIDPRINT("Xlib fully up.\n");

	return GGI_DL_OPDISPLAY;
}

int GGIdlcleanup(ggi_visual *vis)
{
	struct Xhooks *priv = LIBGGI_PRIVATE(vis);

	if (priv->cmap) XFreeColormap(priv->display, priv->cmap);
	if (priv->textfont) XUnloadFont(priv->display, priv->textfont->fid);
	if (priv->gc) XFreeGC(priv->display, priv->gc);
	if (priv->tempgc) XFreeGC(priv->display, priv->tempgc);
	if (priv->window) XDestroyWindow(priv->display, priv->window);
	if (priv->cursor != None) XFreeCursor(priv->display, priv->cursor);
	XSync(priv->display, 0);
	if (priv->display) XCloseDisplay(priv->display);
	
	ggLockDestroy(priv->XLibLock);
	
	free(LIBGGI_PRIVATE(vis));

	if(vis->palette) free(vis->palette);
	free(LIBGGI_GC(vis));

	return 0;
}

#include <ggi/internal/ggidlinit.h>

==> libggi-2.0b1.tar.gz: libggi-2.0b1/display/Xlib/visual.c <==
/* $Id: visual.c,v 1.25 1998/12/18 21:36:22 marcus Exp $
******************************************************************************

   Display-Xlib initialization.

   Copyright (C) 1995 Andreas Beck      [becka@ggi-project.org]
   Copyright (C) 1997 Jason McMullan    [jmcc@ggi-project.org]

   Permission is hereby granted, free of charge, to any person obtaining a
   copy of this software and associated documentation files (the "Software"),
   to deal in the Software without restriction, including without limitation
   the rights to use, copy, modify, merge, publish, distribute, sublicense,
   and/or sell copies of the Software, and to permit persons to whom the
   Software is furnished to do so, subject to the following conditions:

   The above copyright notice and this permission notice shall be included in
   all copies or substantial portions of the Software.

   THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
   IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
   FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT.  IN NO EVENT SHALL
   THE AUTHOR(S) BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
   IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
   CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

******************************************************************************
*/

#include <stdlib.h>

#include <ggi/internal/ggi-dl.h>
#include <ggi/display/xlib.h>

int GGIdlinit(ggi_visual *vis,const char *args,void *argptr)
{
	struct Xhooks *xinfo;

	DPRINT("X-lib starting.\n");

	xinfo = _ggi_malloc(sizeof(struct Xhooks));

	if ((xinfo->XLibLock = ggLockCreate()) == NULL) {
		free(xinfo);
		return GGI_DL_ERROR;
	}

	LIBGGI_GC(vis) = _ggi_malloc(sizeof(ggi_gc));

	xinfo->display=NULL;
	xinfo->cmap=0;
	xinfo->cmap_first=256;
	xinfo->cmap_last=0;
	xinfo->window=0;
	xinfo->gc=0;
	xinfo->tempgc=0;
	xinfo->viswidth=0;
	xinfo->visheight=0;

	DPRINT("X-lib wants display %s.\n",args);
	xinfo->display = XOpenDisplay(args);
	if (xinfo->display == NULL)
		return GGI_DL_ERROR;

	DPRINT("X-lib has display %s.\n",args);

	xinfo->screen = DefaultScreen (xinfo->display);
	DPRINT("X-lib has screen.\n");
	DPRINT("X-lib fully up.\n");

	/* Has mode management */
	vis->opdisplay->flush=GGI_Xlib_flush;
	vis->opdisplay->getmode=GGI_Xlib_getmode;
	vis->opdisplay->setmode=GGI_Xlib_setmode;
	vis->opdisplay->checkmode=GGI_Xlib_checkmode;
	vis->opdisplay->getapi =GGI_Xlib_getapi;
	vis->opdisplay->setflags=GGI_Xlib_setflags;

	{
		gii_inputxwin_arg args;
		gii_input *inp;
		
		args.disp = xinfo->display;
		args.ptralwaysrel = 0;
		args.wait = 1;
		
		if ((inp = giiOpen("xwin", &args, NULL)) == NULL) {
			DPRINT_MISC("Unable to open xwin inputlib\n");
			GGIdlcleanup(vis);
			return GGI_DL_ERROR;
		}

		xinfo->inp = inp;
		/* Now join the new event source in. */
		vis->input = giiJoinInputs(vis->input, inp);
        }

	LIBGGI_PRIVATE(vis) = xinfo;

	return GGI_DL_OPDISPLAY;
}

int GGIdlcleanup(ggi_visual *vis)
{
	struct Xhooks *xinfo;

	if ((xinfo=LIBGGI_PRIVATE(vis)) != NULL) {
		if (xinfo->cmap)
			XFreeColormap(xinfo->display,xinfo->cmap);
		if (xinfo->gc) {
			XFreeGC(xinfo->display,xinfo->gc);
			XFreeGC(xinfo->display,xinfo->tempgc);
		}
		if (xinfo->window) 
			XDestroyWindow(xinfo->display,xinfo->window);
		XSync(xinfo->display,0);
		if (xinfo->display)
		{
			XCloseDisplay(xinfo->display);
		}

		ggLockDestroy(xinfo->XLibLock);

		free(LIBGGI_PRIVATE(vis));
	}

	if(vis->palette)
		free(vis->palette);
	free(LIBGGI_GC(vis));

	return 0;
}

#include <ggi/internal/ggidlinit.h>
